@@ -42,6 +42,21 @@
 
 #include <ttfautohint.h>
 
+
+#ifdef _WIN32
+#  include <unistd.h>
+#  include <fcntl.h>
+#  define SET_BINARY(f) do { \
+                          if (!isatty(fileno(f))) \
+                            setmode(fileno(f), O_BINARY); \
+                        } while (0)
+#endif
+
+#ifndef SET_BINARY
+#  define SET_BINARY(f) do {} while (0)
+#endif
+
+
 using namespace std;
 
 
@@ -69,6 +84,7 @@ progress(long curr_idx,
   {
     fprintf(stderr, "subfont %ld of %ld\n", curr_sfnt + 1, num_sfnts);
     data->last_sfnt = curr_sfnt;
+    data->last_percent = 0;
     data->begin = true;
   }
 
@@ -115,10 +131,13 @@ show_help(bool
 "Usage: ttfautohintGUI [OPTION]...\n"
 "A GUI application to replace hints in a TrueType font.\n"
 #else
-"Usage: ttfautohint [OPTION]... IN-FILE OUT-FILE\n"
+"Usage: ttfautohint [OPTION]... [IN-FILE [OUT-FILE]]\n"
 "Replace hints in TrueType font IN-FILE and write output to OUT-FILE.\n"
+"If OUT-FILE is missing, standard output is used instead;\n"
+"if IN-FILE is missing also, standard input and output are used.\n"
 #endif
-"The new hints are based on FreeType's autohinter.\n"
+"\n"
+"The new hints are based on FreeType's auto-hinter.\n"
 "\n"
 "This program is a simple front-end to the `ttfautohint' library.\n"
 "\n");
@@ -141,6 +160,7 @@ show_help(bool
 #ifndef BUILD_GUI
 "      --debug                print debugging information\n"
 #endif
+"  -c, --components           hint glyph components separately\n"
 "  -f, --latin-fallback       set fallback script to latin\n"
 "  -G, --hinting-limit=N      switch off hinting above this PPEM value\n"
 "                             (default: %d); value 0 means no limit\n"
@@ -297,6 +317,7 @@ main(int argc,
 
   bool ignore_restrictions = false;
   bool pre_hinting = false;
+  bool hint_with_components = true;
   bool no_info = false;
   int latin_fallback = 0; // leave it as int; this probably gets extended
   bool symbol = false;
@@ -334,6 +355,7 @@ main(int argc,
 #endif
 
       // ttfautohint options
+      {"components", no_argument, NULL, 'c'},
 #ifndef BUILD_GUI
       {"debug", no_argument, NULL, DEBUG_OPTION},
 #endif
@@ -381,13 +403,17 @@ main(int argc,
     };
 
     int option_index;
-    int c = getopt_long_only(argc, argv, "fG:hil:npr:stVvx:X:w:",
+    int c = getopt_long_only(argc, argv, "cfG:hil:npr:stVvx:X:w:",
                              long_options, &option_index);
     if (c == -1)
       break;
 
     switch (c)
     {
+    case 'c':
+      hint_with_components = false;
+      break;
+
     case 'f':
       latin_fallback = 1;
       break;
@@ -520,36 +546,56 @@ main(int argc,
                     hinting_range_max);
     exit(EXIT_FAILURE);
   }
-  if (increase_x_height != 0
-      && (increase_x_height < 6 || increase_x_height > 20))
+  if (increase_x_height != 0 && increase_x_height < 6)
   {
     fprintf(stderr, "A non-zero x height increase limit"
-                    " must be in the range 6-20\n");
+                    " must be larger than or equal to 6\n");
     exit(EXIT_FAILURE);
   }
 
-  // on the console we need in and out file arguments
-  if (argc - optind != 2)
+  int num_args = argc - optind;
+
+  if (num_args > 2)
     show_help(false, true);
 
-  FILE* in = fopen(argv[optind], "rb");
-  if (!in)
+  FILE* out;
+  if (num_args > 1)
   {
-    fprintf(stderr, "The following error occurred while opening font `%s':\n"
-                    "\n"
-                    "  %s\n",
-                    argv[optind], strerror(errno));
-    exit(EXIT_FAILURE);
+    out = fopen(argv[optind + 1], "wb");
+    if (!out)
+    {
+      fprintf(stderr, "The following error occurred while opening font `%s':\n"
+                      "\n"
+                      "  %s\n",
+                      argv[optind + 1], strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+  }
+  else
+  {
+    if (isatty(fileno(stdout)))
+      show_help(false, true);
+    out = stdout;
   }
 
-  FILE* out = fopen(argv[optind + 1], "wb");
-  if (!out)
+  FILE* in;
+  if (num_args > 0)
   {
-    fprintf(stderr, "The following error occurred while opening font `%s':\n"
-                    "\n"
-                    "  %s\n",
-                    argv[optind + 1], strerror(errno));
-    exit(EXIT_FAILURE);
+    in = fopen(argv[optind], "rb");
+    if (!in)
+    {
+      fprintf(stderr, "The following error occurred while opening font `%s':\n"
+                      "\n"
+                      "  %s\n",
+                      argv[optind], strerror(errno));
+      exit(EXIT_FAILURE);
+    }
+  }
+  else
+  {
+    if (isatty(fileno(stdin)))
+      show_help(false, true);
+    in = stdin;
   }
 
   unsigned char version_data[128];
@@ -575,6 +621,7 @@ main(int argc,
     info_data.dw_cleartype_strong_stem_width = dw_cleartype_strong_stem_width;
 
     info_data.pre_hinting = pre_hinting;
+    info_data.hint_with_components = hint_with_components;
     info_data.increase_x_height = increase_x_height;
     info_data.latin_fallback = latin_fallback;
     info_data.symbol = symbol;
@@ -582,6 +629,11 @@ main(int argc,
     build_version_string(&info_data);
   }
 
+  if (in == stdin)
+    SET_BINARY(stdin);
+  if (out == stdout)
+    SET_BINARY(stdout);
+
   TA_Error error =
     TTF_autohint("in-file, out-file,"
                  "hinting-range-min, hinting-range-max, hinting-limit,"
@@ -590,8 +642,8 @@ main(int argc,
                  "error-string,"
                  "progress-callback, progress-callback-data,"
                  "info-callback, info-callback-data,"
-                 "ignore-restrictions, pre-hinting, increase-x-height,"
-                 "fallback-script, symbol,"
+                 "ignore-restrictions, pre-hinting, hint-with-components,"
+                 "increase-x-height, fallback-script, symbol,"
                  "debug",
                  in, out,
                  hinting_range_min, hinting_range_max, hinting_limit,
@@ -600,8 +652,8 @@ main(int argc,
                  &error_string,
                  progress_func, &progress_data,
                  info_func, &info_data,
-                 ignore_restrictions, pre_hinting, increase_x_height,
-                 latin_fallback, symbol,
+                 ignore_restrictions, pre_hinting, hint_with_components,
+                 increase_x_height, latin_fallback, symbol,
                  debug);
 
   if (error)
@@ -643,8 +695,10 @@ main(int argc,
     exit(EXIT_FAILURE);
   }
 
-  fclose(in);
-  fclose(out);
+  if (in != stdin)
+    fclose(in);
+  if (out != stdout)
+    fclose(out);
 
   exit(EXIT_SUCCESS);
 
@@ -669,7 +723,7 @@ main(int argc,
                gray_strong_stem_width, gdi_cleartype_strong_stem_width,
                dw_cleartype_strong_stem_width,
                increase_x_height, ignore_restrictions, pre_hinting,
-               no_info, latin_fallback, symbol);
+               hint_with_components, no_info, latin_fallback, symbol);
   gui.show();
 
   return app.exec();
