@@ -24,6 +24,7 @@
 #include FT_TRUETYPE_TAGS_H
 
 #include "taloader.h"
+#include "taglobal.h"
 #include "tadummy.h"
 #include "talatin.h"
 
@@ -125,6 +126,17 @@ typedef struct glyf_Data_
 {
   FT_UShort num_glyphs;
   GLYPH* glyphs;
+
+  /* this index gives the `master' globals for a `glyf' table; */
+  /* see function `TA_sfnt_handle_coverage' */
+  TA_FaceGlobals master_globals;
+
+  /* if a `glyf' table gets used in more than one subfont, */
+  /* so do `cvt', `fpgm', and `prep' tables: */
+  /* these four tables are always handled in parallel */
+  FT_ULong cvt_idx;
+  FT_ULong fpgm_idx;
+  FT_ULong prep_idx;
 } glyf_Data;
 
 /* an SFNT table */
@@ -173,7 +185,7 @@ typedef struct SFNT_
 } SFNT;
 
 /* our font object */
-typedef struct FONT_
+struct FONT_
 {
   FT_Library lib;
 
@@ -191,6 +203,9 @@ typedef struct FONT_
 
   FT_Bool have_DSIG;
 
+  /* we have a single `gasp' table for all subfonts */
+  FT_ULong gasp_idx;
+
   TA_LoaderRec loader[1]; /* the interface to the autohinter */
 
   /* configuration options */
@@ -206,13 +221,14 @@ typedef struct FONT_
   FT_Bool gdi_cleartype_strong_stem_width;
   FT_Bool dw_cleartype_strong_stem_width;
   FT_Bool pre_hinting;
+  FT_Bool hint_with_components;
   FT_Bool no_x_height_snapping;
   FT_Byte* x_height_snapping_exceptions;
   FT_Bool ignore_restrictions;
   FT_UInt fallback_script;
   FT_Bool symbol;
   FT_Bool debug;
-} FONT;
+};
 
 
 #include "tatables.h"
@@ -265,9 +281,6 @@ TA_sfnt_build_gasp_table(SFNT* sfnt,
                          FONT* font);
 
 FT_Error
-TA_sfnt_build_glyf_hints(SFNT* sfnt,
-                         FONT* font);
-FT_Error
 TA_sfnt_split_glyf_table(SFNT* sfnt,
                          FONT* font);
 FT_Error
@@ -276,6 +289,17 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
 FT_Error
 TA_sfnt_create_glyf_data(SFNT* sfnt,
                          FONT* font);
+FT_Error
+TA_sfnt_handle_coverage(SFNT* sfnt,
+                        FONT* font);
+void
+TA_sfnt_adjust_master_coverage(SFNT* sfnt,
+                               FONT* font);
+#if 0
+void
+TA_sfnt_copy_master_coverage(SFNT* sfnt,
+                             FONT* font);
+#endif
 
 FT_Error
 TA_sfnt_update_GPOS_table(SFNT* sfnt,
