@@ -18,7 +18,7 @@
 
 
 #undef MISSING
-#define MISSING (FT_UInt)~0
+#define MISSING (FT_UShort)~0
 
 /* a simple macro to emit bytecode instructions */
 #define BCI(code) *(bufp++) = (code)
@@ -58,22 +58,39 @@ typedef struct Recorder_
   /* (there can be at most one such segment per contour); */
   /* later on we append additional records */
   /* to split them into 24-26 and 0-2 */
-  FT_UInt* wrap_around_segments;
-  FT_UInt num_wrap_around_segments;
+  FT_UShort* wrap_around_segments;
+  FT_UShort num_wrap_around_segments;
 
   FT_UShort num_stack_elements; /* the necessary stack depth so far */
 
   /* data necessary for strong point interpolation */
-  FT_UInt* ip_before_points;
-  FT_UInt* ip_after_points;
-  FT_UInt* ip_on_point_array;
-  FT_UInt* ip_between_point_array;
+  FT_UShort* ip_before_points;
+  FT_UShort* ip_after_points;
+  FT_UShort* ip_on_point_array;
+  FT_UShort* ip_between_point_array;
 
-  FT_UInt num_strong_points;
-  FT_UInt num_segments;
+  FT_UShort num_strong_points;
+  FT_UShort num_segments;
 } Recorder;
 
 
+/* this is the bytecode of the `.ttfautohint' glyph */
+
+FT_Byte ttfautohint_glyph_bytecode[7] =
+{
+
+  /* increment `cvtl_is_subglyph' counter */
+  PUSHB_3,
+    cvtl_is_subglyph,
+    1,
+    cvtl_is_subglyph,
+  RCVT,
+  ADD,
+  WCVTP,
+
+};
+
+
 /* We add a subglyph for each composite glyph. */
 /* Since subglyphs must contain at least one point, */
 /* we have to adjust all point indices accordingly. */
@@ -87,11 +104,12 @@ static FT_UInt
 TA_adjust_point_index(Recorder* recorder,
                       FT_UInt idx)
 {
+  FONT* font = recorder->font;
   GLYPH* glyph = recorder->glyph;
   FT_UShort i;
 
 
-  if (!glyph->num_components)
+  if (!glyph->num_components || !font->hint_with_components)
     return idx; /* not a composite glyph */
 
   for (i = 0; i < glyph->num_pointsums; i++)
@@ -125,17 +143,17 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   FT_UInt* arg;
   FT_UInt num_args;
   FT_UInt nargs;
-  FT_UInt num_segments;
+  FT_UShort num_segments;
 
   FT_Bool need_words = 0;
 
   FT_Int n;
   FT_UInt i, j;
   FT_UInt base;
-  FT_UInt num_packed_segments;
-  FT_UInt num_storage;
-  FT_UInt num_stack_elements;
-  FT_UInt num_twilight_points;
+  FT_UShort num_packed_segments;
+  FT_UShort num_storage;
+  FT_UShort num_stack_elements;
+  FT_UShort num_twilight_points;
 
 
   seg_limit = segments + axis->num_segments;
@@ -188,7 +206,7 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
     need_words = 1;
 
   /* the number of packed segments is indicated by the function number */
-  if (recorder->glyph->num_components)
+  if (recorder->glyph->num_components && font->hint_with_components)
     *(arg--) = bci_create_segments_composite_0 + num_packed_segments;
   else
     *(arg--) = bci_create_segments_0 + num_packed_segments;
@@ -355,6 +373,7 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
                            Recorder* recorder,
                            FT_Byte* bufp)
 {
+  FONT* font = recorder->font;
   FT_GlyphSlot glyph = sfnt->face->glyph;
   FT_Vector* points = glyph->outline.points;
   FT_Int num_contours = glyph->outline.n_contours;
@@ -368,7 +387,7 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   FT_Int p, q;
   FT_UInt i, j;
   FT_Int start, end;
-  FT_UInt num_stack_elements;
+  FT_UShort num_stack_elements;
 
 
   num_args = 2 * num_contours + 2;
@@ -385,7 +404,7 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   if (num_args > 0xFF)
     need_words = 1;
 
-  if (recorder->glyph->num_components)
+  if (recorder->glyph->num_components && font->hint_with_components)
     *(arg--) = bci_scale_composite_glyph;
   else
     *(arg--) = bci_scale_glyph;
@@ -602,19 +621,19 @@ TA_build_point_hints(Recorder* recorder,
   TA_Edge after;
 
   FT_Byte* p = recorder->hints_record.buf;
-  FT_UInt num_edges = axis->num_edges;
-  FT_UInt num_strong_points = recorder->num_strong_points;
+  FT_UShort num_edges = axis->num_edges;
+  FT_UShort num_strong_points = recorder->num_strong_points;
 
-  FT_UInt i;
-  FT_UInt j;
-  FT_UInt k;
+  FT_UShort i;
+  FT_UShort j;
+  FT_UShort k;
 
-  FT_UInt* ip;
-  FT_UInt* iq;
-  FT_UInt* ir;
-  FT_UInt* ip_limit;
-  FT_UInt* iq_limit;
-  FT_UInt* ir_limit;
+  FT_UShort* ip;
+  FT_UShort* iq;
+  FT_UShort* ir;
+  FT_UShort* ip_limit;
+  FT_UShort* iq_limit;
+  FT_UShort* ir_limit;
 
 
   /* we store everything as 16bit numbers; */
@@ -1038,13 +1057,13 @@ static FT_Byte*
 TA_hints_recorder_handle_segments(FT_Byte* bufp,
                                   TA_AxisHints axis,
                                   TA_Edge edge,
-                                  FT_UInt* wraps)
+                                  FT_UShort* wraps)
 {
   TA_Segment segments = axis->segments;
   TA_Segment seg;
-  FT_UInt seg_idx;
-  FT_UInt num_segs = 0;
-  FT_UInt* wrap;
+  FT_UShort seg_idx;
+  FT_UShort num_segs = 0;
+  FT_UShort* wrap;
 
 
   seg_idx = edge->first - segments;
@@ -1133,11 +1152,11 @@ TA_hints_recorder(TA_Action action,
 
   Recorder* recorder = (Recorder*)hints->user;
   FONT* font = recorder->font;
-  FT_UInt* wraps = recorder->wrap_around_segments;
+  FT_UShort* wraps = recorder->wrap_around_segments;
   FT_Byte* p = recorder->hints_record.buf;
 
-  FT_UInt* ip;
-  FT_UInt* limit;
+  FT_UShort* ip;
+  FT_UShort* limit;
 
 
   if (dim == TA_DIMENSION_HORZ)
@@ -1517,8 +1536,8 @@ TA_init_recorder(Recorder* recorder,
   TA_Segment seg_limit = segments + axis->num_segments;
   TA_Segment seg;
 
-  FT_UInt num_strong_points = 0;
-  FT_UInt* wrap_around_segment;
+  FT_UShort num_strong_points = 0;
+  FT_UShort* wrap_around_segment;
 
   recorder->font = font;
   recorder->glyph = glyph;
@@ -1540,7 +1559,8 @@ TA_init_recorder(Recorder* recorder,
       recorder->num_wrap_around_segments++;
 
   recorder->wrap_around_segments =
-    (FT_UInt*)malloc(recorder->num_wrap_around_segments * sizeof (FT_UInt));
+    (FT_UShort*)malloc(recorder->num_wrap_around_segments
+                     * sizeof (FT_UShort));
   if (!recorder->wrap_around_segments)
     return FT_Err_Out_Of_Memory;
 
@@ -1564,12 +1584,12 @@ TA_init_recorder(Recorder* recorder,
   recorder->num_strong_points = num_strong_points;
 
   recorder->ip_before_points =
-    (FT_UInt*)malloc(num_strong_points * sizeof (FT_UInt));
+    (FT_UShort*)malloc(num_strong_points * sizeof (FT_UShort));
   if (!recorder->ip_before_points)
     return FT_Err_Out_Of_Memory;
 
   recorder->ip_after_points =
-    (FT_UInt*)malloc(num_strong_points * sizeof (FT_UInt));
+    (FT_UShort*)malloc(num_strong_points * sizeof (FT_UShort));
   if (!recorder->ip_after_points)
     return FT_Err_Out_Of_Memory;
 
@@ -1577,14 +1597,14 @@ TA_init_recorder(Recorder* recorder,
   /* however, this value isn't known yet */
   /* (or rather, it can vary between different pixel sizes) */
   recorder->ip_on_point_array =
-    (FT_UInt*)malloc(axis->num_segments
-                     * num_strong_points * sizeof (FT_UInt));
+    (FT_UShort*)malloc(axis->num_segments
+                       * num_strong_points * sizeof (FT_UShort));
   if (!recorder->ip_on_point_array)
     return FT_Err_Out_Of_Memory;
 
   recorder->ip_between_point_array =
-    (FT_UInt*)malloc(axis->num_segments * axis->num_segments
-                     * num_strong_points * sizeof (FT_UInt));
+    (FT_UShort*)malloc(axis->num_segments * axis->num_segments
+                       * num_strong_points * sizeof (FT_UShort));
   if (!recorder->ip_between_point_array)
     return FT_Err_Out_Of_Memory;
 
@@ -1613,16 +1633,16 @@ TA_rewind_recorder(Recorder* recorder,
   /* We later check with MISSING (which expands to 0xFF bytes) */
 
   memset(recorder->ip_before_points, 0xFF,
-         recorder->num_strong_points * sizeof (FT_UInt));
+         recorder->num_strong_points * sizeof (FT_UShort));
   memset(recorder->ip_after_points, 0xFF,
-         recorder->num_strong_points * sizeof (FT_UInt));
+         recorder->num_strong_points * sizeof (FT_UShort));
 
   memset(recorder->ip_on_point_array, 0xFF,
          recorder->num_segments
-         * recorder->num_strong_points * sizeof (FT_UInt));
+         * recorder->num_strong_points * sizeof (FT_UShort));
   memset(recorder->ip_between_point_array, 0xFF,
          recorder->num_segments * recorder->num_segments
-         * recorder->num_strong_points * sizeof (FT_UInt));
+         * recorder->num_strong_points * sizeof (FT_UShort));
 }
 
 
@@ -1664,7 +1684,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   Hints_Record* point_hints_records = NULL;
 
   Recorder recorder;
-  FT_UInt num_stack_elements;
+  FT_UShort num_stack_elements;
   FT_Bool optimize = 0;
 
   FT_Int32 load_flags;
@@ -1679,13 +1699,14 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
 
   /* XXX: right now, we abuse this flag to control */
   /*      the global behaviour of the auto-hinter */
-  load_flags = font->fallback_script << 30;
-  load_flags |= 1 << 29; /* vertical hinting only */
-  /* values 0 and 6-20 compressed to 4 bits */
-  if (font->increase_x_height)
-    load_flags |= (font->increase_x_height - 5) << 25;
+  load_flags = 1 << 29; /* vertical hinting only */
   if (!font->pre_hinting)
-    load_flags |= FT_LOAD_NO_SCALE;
+  {
+    if (font->hint_with_components)
+      load_flags |= FT_LOAD_NO_SCALE;
+    else
+      load_flags |= FT_LOAD_NO_RECURSE;
+  }
 
   /* computing the segments is resolution independent, */
   /* thus the pixel size in this call is arbitrary */
@@ -1701,7 +1722,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
 #endif
 
   ta_loader_register_hints_recorder(font->loader, NULL, NULL);
-  error = ta_loader_load_glyph(font->loader, face, (FT_UInt)idx, load_flags);
+  error = ta_loader_load_glyph(font, face, (FT_UInt)idx, load_flags);
 
 #ifdef TA_DEBUG
   _ta_debug = _ta_debug_save;
@@ -1749,7 +1770,8 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   if (font->loader->metrics->clazz == &ta_dummy_script_class)
   {
     /* since `TA_init_recorder' hasn't been called yet, */
-    /* we manually initialize the `glyph' field */
+    /* we manually initialize the `font' and `glyph' fields */
+    recorder.font = font;
     recorder.glyph = glyph;
 
     bufp = TA_sfnt_build_glyph_scaler(sfnt, &recorder, ins_buf);
@@ -1812,7 +1834,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     /* calling `ta_loader_load_glyph' uses the */
     /* `TA_hints_recorder' function as a callback, */
     /* modifying `hints_record' */
-    error = ta_loader_load_glyph(font->loader, face, idx, load_flags);
+    error = ta_loader_load_glyph(font, face, idx, load_flags);
     if (error)
       goto Err;
 
