@@ -26,10 +26,6 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
   FT_Int32 load_flags;
 
 
-  error = ta_loader_init(font->loader);
-  if (error)
-    return error;
-
   error = FT_Select_Charmap(face, FT_ENCODING_UNICODE);
   if (error)
   {
@@ -55,9 +51,8 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
       return TA_Err_Missing_Glyph;
   }
 
-  load_flags = font->fallback_script << 30;
-  load_flags |= 1 << 29; /* vertical hinting only */
-  error = ta_loader_load_glyph(font->loader, face, idx, load_flags);
+  load_flags = 1 << 29; /* vertical hinting only */
+  error = ta_loader_load_glyph(font, face, idx, load_flags);
 
   return error;
 }
@@ -89,7 +84,7 @@ TA_table_build_cvt(FT_Byte** cvt,
   if (error)
     return error;
 
-  if (font->loader->hints.metrics->clazz->script == TA_SCRIPT_NONE)
+  if (font->loader->hints.metrics->clazz->script == TA_SCRIPT_DUMMY)
   {
     haxis = NULL;
     vaxis = NULL;
@@ -203,7 +198,10 @@ FT_Error
 TA_sfnt_build_cvt_table(SFNT* sfnt,
                         FONT* font)
 {
-  FT_Error error;
+  FT_Error error = FT_Err_Ok;
+
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
 
   FT_Byte* cvt_buf;
   FT_ULong cvt_len;
@@ -211,11 +209,18 @@ TA_sfnt_build_cvt_table(SFNT* sfnt,
 
   error = TA_sfnt_add_table_info(sfnt);
   if (error)
-    return error;
+    goto Exit;
+
+  /* `glyf', `cvt', `fpgm', and `prep' are always used in parallel */
+  if (glyf_table->processed)
+  {
+    sfnt->table_infos[sfnt->num_table_infos - 1] = data->cvt_idx;
+    goto Exit;
+  }
 
   error = TA_table_build_cvt(&cvt_buf, &cvt_len, sfnt, font);
   if (error)
-    return error;
+    goto Exit;
 
   /* in case of success, `cvt_buf' gets linked */
   /* and is eventually freed in `TA_font_unload' */
@@ -223,12 +228,12 @@ TA_sfnt_build_cvt_table(SFNT* sfnt,
                             &sfnt->table_infos[sfnt->num_table_infos - 1],
                             TTAG_cvt, cvt_len, cvt_buf);
   if (error)
-  {
     free(cvt_buf);
-    return error;
-  }
+  else
+    data->cvt_idx = sfnt->table_infos[sfnt->num_table_infos - 1];
 
-  return FT_Err_Ok;
+Exit:
+  return error;
 }
 
 /* end of tacvt.c */
