@@ -5111,7 +5111,10 @@ FT_Error
 TA_sfnt_build_fpgm_table(SFNT* sfnt,
                          FONT* font)
 {
-  FT_Error error;
+  FT_Error error = FT_Err_Ok;
+
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
 
   FT_Byte* fpgm_buf;
   FT_ULong fpgm_len;
@@ -5119,11 +5122,18 @@ TA_sfnt_build_fpgm_table(SFNT* sfnt,
 
   error = TA_sfnt_add_table_info(sfnt);
   if (error)
-    return error;
+    goto Exit;
+
+  /* `glyf', `cvt', `fpgm', and `prep' are always used in parallel */
+  if (glyf_table->processed)
+  {
+    sfnt->table_infos[sfnt->num_table_infos - 1] = data->fpgm_idx;
+    goto Exit;
+  }
 
   error = TA_table_build_fpgm(&fpgm_buf, &fpgm_len, font);
   if (error)
-    return error;
+    goto Exit;
 
   if (fpgm_len > sfnt->max_instructions)
     sfnt->max_instructions = fpgm_len;
@@ -5134,12 +5144,12 @@ TA_sfnt_build_fpgm_table(SFNT* sfnt,
                             &sfnt->table_infos[sfnt->num_table_infos - 1],
                             TTAG_fpgm, fpgm_len, fpgm_buf);
   if (error)
-  {
     free(fpgm_buf);
-    return error;
-  }
+  else
+    data->fpgm_idx = sfnt->table_infos[sfnt->num_table_infos - 1];
 
-  return FT_Err_Ok;
+Exit:
+  return error;
 }
 
 /* end of tafpgm.c */
