@@ -22,6 +22,7 @@
 #include <ft2build.h>
 #include FT_ADVANCES_H
 
+#include "taglobal.h"
 #include "talatin.h"
 #include "tasort.h"
 
@@ -596,18 +597,19 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
       FT_Pos threshold;
       FT_Pos fitted;
       FT_UInt limit;
+      FT_UInt ppem;
 
 
       scaled = FT_MulFix(blue->shoot.org, scaler->y_scale);
-
+      ppem = metrics->root.scaler.face->size->metrics.x_ppem;
+      limit = metrics->root.globals->increase_x_height;
       threshold = 40;
-      /* scaler flag bits 3-6 hold the x height increase limit; */
-      /* if zero, the feature is switched off, */
-      /* otherwise the limit is the bits value + 5 */
-      limit = (scaler->flags >> 3) & 15;
+
+      /* if the `increase-x-height' property is active, */
+      /* we round up much more often                    */
       if (limit
-          && metrics->root.scaler.face->size->metrics.x_ppem <= limit + 5
-          && metrics->root.scaler.face->size->metrics.x_ppem >= 6)
+          && ppem <= limit
+          && ppem >= TA_PROP_INCREASE_X_HEIGHT_MIN)
         threshold = 52;
 
       fitted = (scaled + threshold) & ~63;
@@ -2431,10 +2433,12 @@ static const TA_Script_UniRangeRec ta_latin_uniranges[] =
   TA_UNIRANGE_REC(0x2460UL, 0x24FFUL), /* Enclosed Alphanumerics */
   TA_UNIRANGE_REC(0x2C60UL, 0x2C7FUL), /* Latin Extended-C */
   TA_UNIRANGE_REC(0x2DE0UL, 0x2DFFUL), /* Cyrillic Extended-A */
+  TA_UNIRANGE_REC(0x2E00UL, 0x2E7FUL), /* Supplemental Punctuation */
   TA_UNIRANGE_REC(0xA640UL, 0xA69FUL), /* Cyrillic Extended-B */
   TA_UNIRANGE_REC(0xA720UL, 0xA7FFUL), /* Latin Extended-D */
   TA_UNIRANGE_REC(0xFB00UL, 0xFB06UL), /* Alphab. Present. Forms (Latin Ligs) */
   TA_UNIRANGE_REC(0x1D400UL, 0x1D7FFUL), /* Mathematical Alphanumeric Symbols */
+  TA_UNIRANGE_REC(0x1F100UL, 0x1F1FFUL), /* Enclosed Alphanumeric Supplement */
   TA_UNIRANGE_REC(0UL, 0UL)
 };
 
