@@ -13,25 +13,31 @@
  */
 
 
+#include <string.h>
+
 #include "ta.h"
 
 
-#define TTFAUTOHINT_GLYPH "\x0C.ttfautohint" /* first byte is string length */
-#define TTFAUTOHINT_GLYPH_LEN 13
-
 FT_Error
 TA_sfnt_update_post_table(SFNT* sfnt,
                           FONT* font)
 {
-  SFNT_Table* post_table = &font->tables[sfnt->post_idx];
-  FT_Byte* buf = post_table->buf;
-  FT_ULong buf_len = post_table->len;
+  SFNT_Table* post_table;
+  FT_Byte* buf;
+  FT_ULong buf_len;
   FT_Byte* buf_new;
   FT_ULong buf_new_len;
 
   FT_ULong version;
 
 
+  if (sfnt->post_idx == MISSING)
+    return TA_Err_Ok;
+
+  post_table = &font->tables[sfnt->post_idx];
+  buf = post_table->buf;
+  buf_len = post_table->len;
+
   if (post_table->processed)
     return TA_Err_Ok;
 
@@ -118,7 +124,7 @@ TA_sfnt_update_post_table(SFNT* sfnt,
     memcpy(p_new, p, buf + buf_len - p); /* names */
     p_new += buf + buf_len - p;
 
-    strncpy((char*)p_new, TTFAUTOHINT_GLYPH,
+    strncpy((char*)p_new, TTFAUTOHINT_GLYPH_FIRST_BYTE TTFAUTOHINT_GLYPH,
             TTFAUTOHINT_GLYPH_LEN); /* new entry */
 
     free(buf);
