@@ -53,7 +53,7 @@
   } while (0)
 
 void
-_ta_message(const char *format,
+_ta_message(const char* format,
             ...);
 
 extern int _ta_debug;
@@ -125,13 +125,6 @@ typedef struct TA_GlyphHintsRec_* TA_GlyphHints;
 #define TA_SCALER_FLAG_NO_VERTICAL 0x02 /* disable vertical hinting */
 #define TA_SCALER_FLAG_NO_ADVANCE 0x04 /* disable advance hinting */
 
-/* for ttfautohint, we add more scaler flags; */
-/* note that FreeType will use a different mechanism */
-/* to pass control arguments to the auto-hinter */
-/* (this isn't implemented yet in version 2.4.9) */
-
-/* currently, bits 3-6 hold the x height increase limit */
-
 typedef struct TA_ScalerRec_
 {
   FT_Face face; /* source font face */
@@ -173,7 +166,7 @@ typedef struct TA_ScalerRec_
 
 typedef enum TA_Script_
 {
-  TA_SCRIPT_NONE = 0,
+  TA_SCRIPT_DUMMY = 0,
   TA_SCRIPT_LATIN = 1,
   TA_SCRIPT_CJK = 2,
   TA_SCRIPT_INDIC = 3, 
@@ -188,12 +181,15 @@ typedef enum TA_Script_
 } TA_Script;
 
 typedef struct TA_ScriptClassRec_ const* TA_ScriptClass;
+typedef struct TA_FaceGlobalsRec_* TA_FaceGlobals;
 
 typedef struct TA_ScriptMetricsRec_
 {
   TA_ScriptClass clazz;
   TA_ScalerRec scaler;
   FT_Bool digits_have_same_width;
+
+  TA_FaceGlobals globals; /* to access properties */
 } TA_ScriptMetricsRec, *TA_ScriptMetrics;
 
 
