@@ -68,7 +68,8 @@ TTF_autohint(const char* options,
 
   FT_Bool ignore_restrictions = 0;
   FT_Bool pre_hinting = 0;
-  FT_UInt fallback_script = 0;
+  FT_Bool hint_with_components = 0;
+  FT_UInt fallback_script = TA_SCRIPT_FALLBACK;
   FT_Bool symbol = 0;
 
   FT_Bool debug = 0;
@@ -130,6 +131,8 @@ TTF_autohint(const char* options,
       hinting_range_max = (FT_Long)va_arg(ap, FT_UInt);
     else if (COMPARE("hinting-range-min"))
       hinting_range_min = (FT_Long)va_arg(ap, FT_UInt);
+    else if (COMPARE("hint-with-components"))
+      hint_with_components = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("ignore-restrictions"))
       ignore_restrictions = (FT_Bool)va_arg(ap, FT_Int);
     else if (COMPARE("in-buffer"))
@@ -150,15 +153,15 @@ TTF_autohint(const char* options,
     }
     else if (COMPARE("increase-x-height"))
       increase_x_height = (FT_Long)va_arg(ap, FT_UInt);
+    else if (COMPARE("info-callback"))
+      info = va_arg(ap, TA_Info_Func);
+    else if (COMPARE("info-callback-data"))
+      info_data = va_arg(ap, void*);
     else if (COMPARE("out-buffer"))
     {
       out_file = NULL;
       out_bufp = va_arg(ap, char**);
     }
-    else if (COMPARE("info-callback"))
-      info = va_arg(ap, TA_Info_Func);
-    else if (COMPARE("info-callback-data"))
-      info_data = va_arg(ap, void*);
     else if (COMPARE("out-buffer-len"))
     {
       out_file = NULL;
@@ -178,6 +181,11 @@ TTF_autohint(const char* options,
       progress_data = va_arg(ap, void*);
     else if (COMPARE("symbol"))
       symbol = (FT_Bool)va_arg(ap, FT_Int);
+    else
+    {
+      error = TA_Err_Unknown_Argument;
+      goto Err1;
+    }
 
     /*
       x-height-snapping-exceptions
@@ -239,9 +247,7 @@ TTF_autohint(const char* options,
   if (hinting_limit < 0)
     hinting_limit = TA_HINTING_LIMIT;
 
-  /* we have 4 bits to store the values 0 and 6-20 */
-  if (increase_x_height > 0
-      && (increase_x_height < 6 || increase_x_height > 20))
+  if (increase_x_height > 0 && increase_x_height < 6)
   {
     error = FT_Err_Invalid_Argument;
     goto Err1;
@@ -265,10 +271,12 @@ TTF_autohint(const char* options,
 
   font->ignore_restrictions = ignore_restrictions;
   font->pre_hinting = pre_hinting;
-  /* restrict value to two bits */
-  font->fallback_script = fallback_script & 3;
+  font->hint_with_components = hint_with_components;
+  font->fallback_script = fallback_script;
   font->symbol = symbol;
 
+  font->gasp_idx = MISSING;
+
   font->debug = debug;
 
   /* dump parameters */
@@ -291,6 +299,8 @@ TTF_autohint(const char* options,
          font->hinting_range_max);
     DUMP("hinting-range-min",
          font->hinting_range_min);
+    DUMP("hint-with-components",
+         font->hint_with_components);
     DUMP("ignore-restrictions",
          font->ignore_restrictions);
     DUMP("increase-x-height",
@@ -327,7 +337,10 @@ TTF_autohint(const char* options,
   if (error)
     goto Err;
 
-  /* loop over subfonts */
+  if (font->debug)
+    _ta_debug = 1;
+
+  /* we do some loops over all subfonts */
   for (i = 0; i < font->num_sfnts; i++)
   {
     SFNT* sfnt = &font->sfnts[i];
@@ -337,7 +350,8 @@ TTF_autohint(const char* options,
     error = FT_New_Memory_Face(font->lib, font->in_buf, font->in_len,
                                i, &sfnt->face);
 
-    /* assure that the font hasn't been already processed by ttfautohint */
+    /* assure that the font hasn't been already processed by ttfautohint; */
+    /* another, more thorough check is done in TA_glyph_parse_simple */
     idx = FT_Get_Name_Index(sfnt->face, TTFAUTOHINT_GLYPH);
     if (idx)
     {
@@ -359,6 +373,10 @@ TTF_autohint(const char* options,
     if (error)
       goto Err;
 
+    error = TA_sfnt_handle_coverage(sfnt, font);
+    if (error)
+      goto Err;
+
     /* check permission */
     if (sfnt->OS2_idx != MISSING)
     {
@@ -375,35 +393,28 @@ TTF_autohint(const char* options,
     }
   }
 
-  /* build `gasp' table */
-  error = TA_sfnt_build_gasp_table(&font->sfnts[0], font);
-  if (error)
-    goto Err;
+  for (i = 0; i < font->num_sfnts; i++)
+  {
+    SFNT* sfnt = &font->sfnts[i];
 
-  /* XXX handle subfonts for bytecode tables */
 
-  if (font->debug)
-    _ta_debug = 1;
+    TA_sfnt_adjust_master_coverage(sfnt, font);
+  }
 
-  /* build `cvt ' table */
-  error = TA_sfnt_build_cvt_table(&font->sfnts[0], font);
-  if (error)
-    goto Err;
+#if 0
+  /* this code is here for completeness -- */
+  /* right now, `glyf' tables get hinted only once, */
+  /* and referring subfonts simply reuse it, */
+  /* but this might change in the future */
 
-  /* build `fpgm' table */
-  error = TA_sfnt_build_fpgm_table(&font->sfnts[0], font);
-  if (error)
-    goto Err;
+  for (i = 0; i < font->num_sfnts; i++)
+  {
+    SFNT* sfnt = &font->sfnts[i];
 
-  /* build `prep' table */
-  error = TA_sfnt_build_prep_table(&font->sfnts[0], font);
-  if (error)
-    goto Err;
 
-  /* hint the glyphs and build bytecode */
-  error = TA_sfnt_build_glyf_hints(&font->sfnts[0], font);
-  if (error)
-    goto Err;
+    TA_sfnt_copy_master_coverage(sfnt, font);
+  }
+#endif
 
   /* loop again over subfonts */
   for (i = 0; i < font->num_sfnts; i++)
@@ -411,18 +422,46 @@ TTF_autohint(const char* options,
     SFNT* sfnt = &font->sfnts[i];
 
 
+    error = ta_loader_init(font);
+    if (error)
+      goto Err;
+
+    error = TA_sfnt_build_gasp_table(sfnt, font);
+    if (error)
+      goto Err;
+    error = TA_sfnt_build_cvt_table(sfnt, font);
+    if (error)
+      goto Err;
+    error = TA_sfnt_build_fpgm_table(sfnt, font);
+    if (error)
+      goto Err;
+    error = TA_sfnt_build_prep_table(sfnt, font);
+    if (error)
+      goto Err;
     error = TA_sfnt_build_glyf_table(sfnt, font);
     if (error)
       goto Err;
     error = TA_sfnt_build_loca_table(sfnt, font);
     if (error)
       goto Err;
+
+    if (font->loader)
+      ta_loader_done(font);
+  }
+
+  for (i = 0; i < font->num_sfnts; i++)
+  {
+    SFNT* sfnt = &font->sfnts[i];
+
+
     error = TA_sfnt_update_maxp_table(sfnt, font);
     if (error)
       goto Err;
 
     /* we add one glyph for composites */
-    if (sfnt->max_components && !font->pre_hinting)
+    if (sfnt->max_components
+        && !font->pre_hinting
+        && font->hint_with_components)
     {
       error = TA_sfnt_update_hmtx_table(sfnt, font);
       if (error)
